from .MPI import Intracomm
from typing import Optional
from typing import Sequence

def helloworld(comm: Intracomm, args: Optional[Sequence[str]] = None, verbose: bool = True) -> str: ...
def ringtest(comm: Intracomm, args: Optional[Sequence[str]] = None, verbose: bool = True) -> float: ...
def main(args: Optional[Sequence[str]] = ...) -> None: ...
